﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;


namespace TRsSplineMaster
{
    class MyUtils
    {
        public static int clamp(int p, int l, int h)
        {
            p = (p < l) ? l : (p > h) ? h : p;

            return p;
        }

        public static Point clampP(Point p, Rectangle r)
        {
            p.X= (p.X < r.Left) ? r.Left:(p.X > r.Right) ? r.Right: p.X;
            p.Y = (p.Y < r.Top) ? r.Top : (p.Y > r.Bottom) ? r.Bottom : p.Y;
            return p;
        }

        public static double Pythag(double p1,double p2)
        {
            return Math.Sqrt(p1 * p1 + p2 * p2);
        }

        public static double PythagP(Point p1, Point p2)
        {
            return Math.Sqrt((p2.X - p1.X) *(p2.X - p1.X) + (p2.Y - p1.Y) *(p2.Y - p1.Y));
        }

        public static Point[] RemoveP(Point[] source, int index)
        {
            Point[] dest = new Point[source.Length - 1];
            if ((index > -1) && (index < source.Length))
            {
                Array.Copy(source, 0, dest, 0, index);
                Array.Copy(source, index + 1, dest, index, source.Length - index - 1);
                return dest;
            }
            else
            {
                return source;
            }
        }
        public static Point[] InsertP(Point[] source, int index, Point p)
        {
            Point[] dest = new Point[source.Length + 1];
            if ((index > -1) && (index < source.Length - 1))
            {
                Array.Copy(source, 0, dest, 1, source.Length);
                Array.Copy(source, 0, dest, 0, index + 1);
                dest[index + 1] = p;
                return dest;
            }
            else
            {
                return source;
            }

        }

        public static Point Centroid(Point[] source)
        {
            Point tmp = new Point();
            if (source.Length == 0) return tmp;
            foreach (Point i in source)
            {
                tmp.X += i.X;
                tmp.Y += i.Y;
            }
            tmp.X /= source.Length;
            tmp.Y /=source.Length;
            return tmp;
        }

        public static Point[] ShiftArray(Point[] source, int x,int y)
        {
            if (source.Length == 0) return source;
            Point[] dest = new Point[source.Length];
            Array.Copy(source, dest, source.Length);
            for (int i= 0;i < source.Length;i++)
            {
                dest[i].X += x;
                dest[i].Y += y;
            }
            return dest;
        }
        public static Point[] ScaleArray(Point[] source, Point center, double x, double y)
        {
            if (source.Length == 0) return source;
            Point[] dest = new Point[source.Length];
            Array.Copy(source, dest, source.Length);
            for (int i = 0; i < source.Length; i++)
            {
                dest[i].X = (int)((dest[i].X - center.X) * x + center.X);
                dest[i].Y = (int)((dest[i].Y - center.Y) * y + center.Y);
            }
            return dest;
        }

        public static Point[] RotateArray(Point[] source, Point center, double Pre, double Post)
        {
            if (source.Length == 0) return source;
            Point[] dest = new Point[source.Length];
            Array.Copy(source, dest, source.Length);
            for (int i = 0; i < source.Length; i++)
            {
                dest[i].X = center.X + (int)(Math.Cos(Post - Pre) * (source[i].X - center.X) + Math.Sin(Post - Pre) * (source[i].Y - center.Y));
                dest[i].Y = center.Y + (int)(Math.Cos(Post - Pre) * (source[i].Y - center.Y) - Math.Sin(Post - Pre) * (source[i].X - center.X));

            }
            return dest;
        }

    }
}
